%LSD_EXAMPLE.m 
%
%Please see Booth, A.M., Lamb, M.P., Avouac, J.-P., and Delacourt, C. 
%(2013), Landslide velocity, thickness, and rheology from remote sensing: 
%La Clapiere lansdlide, France", GEOPHYSICAL RESEARCH LETTERS, 40, 16, 
%doi:10.1002/grl.50828.
%
%This script walks through the processes of inverting a landslide's
%velocity field and change in elevation field for its thickness, using 
%synthetic data for a simple slump type landslide.
%
%Copyright (C) 2013 Adam M. Booth, [bootha@caltech.edu].
%
%-------------------------------------------------------------------------%
%This program is free software: you can redistribute it and/or modify
%it under the terms of the GNU General Public License as published by
%the Free Software Foundation, either version 3 of the License, or
%any later version.
%
%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License
%along with this program.  If not, see <http://www.gnu.org/licenses/>.
%-------------------------------------------------------------------------%

%Load the velocity and elevation change data:
load LSD_DATA;      %x and y are coordinates in meters, u and v are the x- 
                    %and y-components of the horizontal velocity field,
                    %respectively, in m/yr, dzdt is the change in
                    %elevation field in m/yr, mask is a matrix of ones and
                    %zeros indicating the active landslide, and h_actual is
                    %the actual landslide thickness.  Noise with a standard
                    %deviation of 0.25 has been added to the dzdt, u, and v
                    %fields.

%Image the change in elevation field, as in Figure 1c:
figure; imagesc(mask.*dzdt); axis equal tight; colorbar;
 
%-----------------------------Part I--------------------------------------%
%Resample u, v, and dzdt repeatedly to a 5 m grid to generate 25
%independent realizations of the 1 m resolution data:

xmin = min(x(:)); xmax = max(x(:));     %Minimum and maximum coordinates
ymin = min(y(:)); ymax = max(y(:));     %of the grid.

nrows = 21; ncols = 21; num = 25;       %Sizes for downsampled grids.
dzdt_stack = zeros(nrows,ncols,num);    %Pre-allocates 3D arrays for
u_stack = zeros(nrows,ncols,num);       %downsampled grids.
v_stack = zeros(nrows,ncols,num);
mask_stack = zeros(nrows,ncols,num);

xvec = 1:5;                             
yvec = 1:5; 
subs = zeros(num,2);
k = 0;
for i = 1:5                            %Creates a matrix of all row
    for j = 1:5                        %and column indices.
        k = k + 1;
        subs(k,1:2) = [xvec(i) yvec(j)];
    end
end
clear i j k;

matlabpool open         %Do the rest in parallel if you can, otherwise
                        %change to regular "for" loops below.
                        
%This is the downsampling loop that fills in the "..._stack" arrays: 
parfor ind = 1:num
    i = subs(ind,1);                                        %Shifts grid by
    j = subs(ind,2);                                        %one pixel.
    [x5,y5] = meshgrid(xmin-1+i:5:xmax-1+i,ymin-1+j:5:ymax-1+j);
    y5 = flipud(y5);                                        %Positive up.
    mask5 = interp2(x,y,mask,x5,y5,'nearest');              %Interpolates
    u5 = mask5.*(interp2(x,y,u,x5,y5,'nearest'));           %everything to    
    v5 = mask5.*(interp2(x,y,v,x5,y5,'nearest'));           %shifted 5m
    dzdt5 = mask5.*interp2(x,y,dzdt,x5,y5,'nearest');       %grid.
    x_stack(:,:,ind) = x5;  
    y_stack(:,:,ind) = y5;
    mask_stack(:,:,ind) = mask5;
    u_stack(:,:,ind) = u5;
    v_stack(:,:,ind) = v5;
    dzdt_stack(:,:,ind) = dzdt5;
end
clear mask5 u5 v5 dzdt5 ind x5 y5;          %Deletes temporary variables.
mask_stack(isnan(mask_stack)) = 0;          %Replaces no data with zero at
u_stack(isnan(u_stack)) = 0;                %the edges of the grids.
v_stack(isnan(v_stack)) = 0;
dzdt_stack(isnan(dzdt_stack)) = 0;

%The mean of u_stack and v_stack gives the mean velocity field as in 
%Figure 1b:
figure; 
quiver(mean(x_stack,3),mean(y_stack,3),mean(u_stack,3),mean(v_stack,3));
axis equal tight;

%---------------------------------Part 2----------------------------------%
%Invert each of the downsampled velocity and dz/dt fields for landslide 
%thickness.  This part assumes that the surface velocity is equal to the
%depth averaged velocity (a sliding block), and Part 3 relaxes this
%assumption by tuning the thickness model to independent data.

%Estimate the uncertianty on the solution vector (b = -2*dx*dz/dt) in order 
%to apply the discrepancy principle for selecting the damping parameter:
dx = 5;
b = -2*dx*dzdt;     
b_stack = interp2(x,y,b,x_stack,y_stack,'nearest');
sigma_b = nanstd(b_stack,[],3);
sigma_b = nanmean(sigma_b(:));          %Averages over study area.
%Note that this estimate of sigma_b is approximately equal to the known
%value of sigma_b = 2.5 since noise with a standard deviation of 0.25 was
%added to the original change in elevation data.

%Find the appropriate misfit norm for each case of downsampled data.  This 
%varies slightly among the 25 downsamplings because the number of pixels 
%on the active landslide varies in each downsampling:
res_tik = zeros(num,1);
for i = 1:num
    res_tik(i) = sigma_b*sqrt(sum(sum(mask_stack(:,:,i))));
end

aspan = 10.^linspace(-3,1,16);          %Vector of damping parameters.
hbest = zeros(nrows,ncols,num);         %Pre-allocates the thickness array,
abest = zeros(num,1);                   %the damping parameter, alpha,
resbest = zeros(num,1);                 %and the misfit norm , ||A*h - b||.
Lres = zeros(length(aspan),1);          %Pre-allocate vectors for the L-
Lbeta = zeros(length(aspan),1);         %curve (||A*h-b|| vs. ||del2(h)||)
htemp = zeros(nrows,ncols,length(aspan));   %Pre-allocates temp thickness.

%This loop estimates the preferred thickness model for each of the 25
%downsamplings, and also keeps track of the optimal damping parameters and 
%misfit norms:
for ind = 1:num
    %Build the matrix A from the downsampled velocity data:
    [A,~,~,~] = lsd_Asvd(u_stack(:,:,ind),v_stack(:,:,ind),0);
    %Loop through each value of the damping parameter in aspan to get the
    %thickness (htemp), misfit norm (Lres), and model norm (Lbeta).  A plot
    %of the misfit norm vs. the model norm is the L-curve:
    parfor i = 1:length(aspan)
        [htemp(:,:,i),Lres(i),Lbeta(i)] = ...
         lsd_smooth(A,dzdt_stack(:,:,ind),mask_stack(:,:,ind),dx,aspan(i));
    end
    %Interpolate to get the damping parameter specified by the discrepency
    %principle:
    abest(ind) = interp1(Lres,aspan,res_tik(ind),'linear',0);
    %Determine h using the optimal damping parameter:
    [hbest(:,:,ind),resbest(ind),~] = ...
       lsd_smooth(A,dzdt_stack(:,:,ind),mask_stack(:,:,ind),dx,abest(ind));
end
clear A ind i htemp Lres Lbeta;             %Deletes temporary variables.

%The optimal thickness model is the mean of the 25 individual models, and
%its uncertainty is the standard deviation.  This analysis has assumed that
%f = 1 (the landslide is a sliding block):
hmean_f1 = nanmean(hbest,3);
hstd_f1 = nanstd(hbest,[],3);
amean = nanmean(abest);             %Mean damping parameter.
resmean = nanmean(resbest);         %Mean misfit norm.
figure; imagesc(hmean_f1); axis equal tight; colorbar;
figure; imagesc(hstd_f1); axis equal tight; colorbar;

%----------------------------------Part 3---------------------------------%
%Tune the thickness model to match independent estimates of thickness
%and/or landslide rheology.  This relaxes the assumption that f = 1 and
%instead finds the value of f that minimizes the misfit between the
%inversion predicted and independently estimated thicknesses:

%Assume we know the depth at three locations on the landslide:
meas_h = [10; 20; 6];
meas_ind = [138; 220; 328];

%This is the short way to find the optimal f, which uses the analytical 
%prediction that h ~ 1/f. (In practice the numerical implementation differs 
%slightly):
fvec = linspace(0.1,1,1000);            %Vector of f values.
n = length(fvec);
RMSerr = zeros(n,1);                    %Pre-allocates misfit vector.
for i = 1:n
    htemp = hmean_f1/fvec(i);
    RMSerr(i) = norm(htemp(meas_ind) - meas_h);
end
figure; plot(fvec,RMSerr);              %Plots the misfit curve.
fbest_an = fvec(RMSerr == min(RMSerr)); %The optimal thickness model is
hmean_an = hmean_f1/fbest_an;           %that which minimizes the misfit.    
hstd_an = hstd_f1/fbest_an;
figure; imagesc(hmean_an); axis equal tight; colorbar;

%Alternatively, find the optimal f numerically, as done in the paper:
fvec = [0.1:0.1:1]';
n = length(fvec);
hbest = zeros(nrows,ncols,n);
hstd = zeros(nrows,ncols,n);
RMSerr = zeros(n,1);
parfor i = 1:n
    f = fvec(i);
    htemp = zeros(nrows,ncols,num);
    for ind = 1:25
        a = abest(ind);
        if isnan(a) == 0
            [A,~,~,~] = lsd_Asvd(f*u_stack(:,:,ind),f*v_stack(:,:,ind),0);
            [htemp(:,:,ind),~,~] = ...
                lsd_smooth(A,dzdt_stack(:,:,ind),mask_stack(:,:,ind),dx,a);
        else
            htemp(:,:,ind) = NaN;
        end
    end
    hmean = nanmean(htemp,3);
    hbest(:,:,i) = hmean;
    hstd(:,:,i) = nanstd(htemp,[],3);
    RMSerr(i) = norm(hmean(meas_ind) - meas_h); 
end
clear A htemp f a i ind hmean;
figure; plot(fvec,RMSerr);              %Plots the misfit curve.
ind = RMSerr == min(RMSerr);            %The optimal thickness model is
hmean_num = hbest(:,:,ind);             %that which minimizes the misfit.
hstd_num = hstd(:,:,ind);
clear ind n;
%Plot the optimal thickness model and its uncertainty.  These are the same 
%as Figure 2a and b in the paper:
figure; imagesc(hmean_num); axis equal tight; colorbar;
figure; imagesc(hstd_num); axis equal tight; colorbar;

%---------------------------------Part 4----------------------------------%
%Explore a range of values for alpha and f to see how sensitive the model
%is to each of these parameters.    

[agrid,fgrid] = ...                                         %Grids of f 
    meshgrid(10.^linspace(-2,0,10),linspace(0.1,1,10));     %and alpha.
n = size(agrid); n = n(1)*n(2);                             

hbest = zeros(nrows,ncols,n);   %Initiates outputs.
hstd = zeros(nrows,ncols,n);
chisq = zeros(n,1);
vol = zeros(n,1);
%NOTE: THIS TAKES A WHILE TO RUN.
parfor i = 1:n                  %Loops through each combination of alpha
    f = fgrid(i);               %and f and keeps track of the landslide 
    a = agrid(i);               %volume and misfit with thicknesses from
    h = zeros(nrows,ncols,num); %Gaffet et al. [2010] (Takes ~ 14.5 hrs).
    for ind = 1:25
        [A,~,~,~] = lsd_Asvd(f*u_stack(:,:,ind),f*v_stack(:,:,ind),0);
        [h(:,:,ind),~,~] = ...
            lsd_smooth(A,dzdt_stack(:,:,ind),mask_stack(:,:,ind),dx,a);
    end
    hmean = nanmean(h,3);
    hbest(:,:,i) = hmean;
    hstd(:,:,i) = nanstd(h,[],3);
    vol(i) = dx*dx*sum(sum(hmean));                     %Landslide volume
    chisq(i) = norm(hmean(meas_ind) - meas_h);          %and misfit.   
end
clear n A h f a i ind hmean;
vol = reshape(vol,length(agrid(:,1)),length(agrid(1,:)));       %Reshapes
chisq = reshape(chisq,length(agrid(:,1)),length(agrid(1,:)));   %to grid.

%Figure 3 is a contour plot of volume and misfit over the ranges of alpha
%and f:
figure; [~,han] = contour(log10(agrid),fgrid,vol,10,'-k');
set(han,'ShowText','on');               %Labels the volume contours.
hold on; contour(log10(agrid),fgrid,chisq,20,':k');

%If you want to view any of the other thickness models, they are stored in
%the 3D array "hbest".

save LSD_DATA_FINAL;                    %Saves entire workspace.